// This code controls an automatic plant watering device. Data from a soil moisture sensor is      // used to determine the soil moisture concentration. If the soil moisture
// concentration is low (less than 40%), the servo motor is held in one position(in which the plant // will not be watered). Otherwise, the servo motor will be moved to a
// position in which the plant will be watered. Additionally, when the moisture concentration is     // low(below 40%), the pin connected to the green LED is high and the 
// pin connected to the red LED is low. When the moisture concentration is too high(over 65%), // then the red LED is high and the green LED is low. Otherwise, the pins
// connected to both LEDs are low. 

int soilMoisture = 0; // initialize soil moisture variable
int AirValue = 575; // experimentally gathered sensor output in air, INPUT YOUR OWN
int WaterValue = 315; // experimentally gathered sensor output in water, INPUT YOUR OWN

#include <Servo.h>
Servo myservo;
int angle = 0;
int dt = 10; 

void setup()
{
  pinMode(12, OUTPUT); // green LED as output
  pinMode(13, OUTPUT);  // red LED as output
  digitalWrite(12, LOW); // set green LED low
  digitalWrite(13, LOW); // set red LED low
  myservo.attach(11); // tell the Arduino which pin will drive the servo 
  Serial.begin(9600);
}

void loop()
{
  soilMoisture = sensor_to_conc(analogRead(0)); // gather sensor data and convert to moisture // concentration
  if (soilMoisture < 40){ // if soil moisture is too dry (under 40%)
    while (angle < 45) { // move to the open position to water the plant
      angle++;        // increment angle 
      myservo.write(angle); // move to new angle 
      delay(dt);
    }
    digitalWrite(12, HIGH); // set green LED high
    digitalWrite(13, LOW); // set red LED low
  }
  else { //
    while (angle > 0) { // move to closed position (plant is not being watered)
      angle--;        // decrement angle 
      myservo.write(angle); // move to new angle 
      delay(dt);
    }
    if (soilMoisture > 65) { // if soil moisture is too high (over 65%)
      digitalWrite(13, HIGH); // set red LED high
    }
    else {
      digitalWrite(13, LOW); // set red LED low
     }
    digitalWrite(12, LOW); // set green LED low
  }
}

// sensor_to_conc takes in raw data from the soil moisture sensor and outputs a value on the    // range 0-100, where 0 symbolizes 0% moisture concentration and 100 represents 100%          // moisture concentration.
int sensor_to_conc (int x) {
 x = x - AirValue; // shift so AirValue maps to 0
 x = 100*x/(WaterValue - AirValue); // normalize to a 0-100 scale
 return x; // return x 
}
